/* @pjs pauseOnBlur=true; */
//=============================================================================
//  carte de champs (chp E, lignes de champ, equipotentielle, carte d'intensité) pour un dipole
//  ©J.Roussel -- 2 Nov 2015
//========== IDEES ==================

int OX, OY;//Origine du dipole
ArrayList lignes,equipot;
boolean afficheMagnitude=false;
boolean afficheControl=true;



void setup() {
  //size(350, 550);
  size(360,360);
  
  smooth();
  ellipseMode(CENTER);
  OX=width/2;
  OY=height/2;
  lignes = new ArrayList();
  equipot= new ArrayList();
 }

void draw() {
  background(64); 
  drawChampE();//carte de champs
  //tracé des lignes de champ
  for (int i = lignes.size()-1; i >= 0; i--) { 
    LigneDeChamp LDC = (LigneDeChamp) lignes.get(i);
    LDC.display();
  }
  for (int i = equipot.size()-1; i >= 0; i--) { 
    Equipotentielle EQP = (Equipotentielle) equipot.get(i);
    EQP.display();
  }
  if (afficheMagnitude) {
    map_champE();
  }//carte d'intensité

  draw_charge();//affichage des charges
  if(afficheControl){affiche_control(0);}
}

void draw_charge() {
  fill(255);
  ellipse(OX, OY, 50, 50);
  stroke(64);
  fill(64);
  pushMatrix();
  translate(OX, OY+10);
  line(0, 0, 0, -10);
  translate(0, -10);
  rotate(-PI/2);
  beginShape(TRIANGLES);
  vertex(0, 3);
  vertex(8, 0);
  vertex(0, -3);
  endShape();
  popMatrix();
}

//champ vectoriel proportionnel au champ E (pour les lignes de champ)
// ATTENTION ! FAIRE&nbsp;: x=xx-OX et  y=yy-OY
PVector champ(float xx, float yy) {
  return new PVector(-3*(xx-OX)*(yy-OY), -2*sq(yy-OY)+sq(xx-OX));
}

//Champ E(x,y) (en pixel par frame)
// ATTENTION ! FAIRE&nbsp;: x=xx-OX et  y=yy-OY
PVector champE(float xx, float yy) {
  float rr;
  rr=pow((sq(xx-OX)+sq(yy-OY)),2);
  //rr=(sq(xx-OX)+sq(yy-OY));  
  if ((dist(xx, yy, OX, OY)<10)) {
    return new PVector(0, 0);
  } else {
    return new PVector(-3*(xx-OX)*(yy-OY)/rr, (-2*sq(yy-OY)+sq(xx-OX))/rr);
  }
}



//Carte de champ
void drawChampE() {  
  float magnitude;
  strokeWeight(1);
  stroke(153);  
  float arrowsize = 4;
  float len=15;
  for (int i=0; i<width; i+=30)
    for (int j=0; j<height; j+=30) {
      if (champ(i, j).mag()!=0) {
        pushMatrix();// Translate to location to render vector
        translate(i, j);// Vector heading to get direction (pointing up is a heading of 0)
        rotate(champ(i, j).heading2D()); // Scale it to be bigger or smaller if necessary
        // Draw three lines to make an arrow
        line(0, 0, len, 0);
        line(len, 0, len-arrowsize, +arrowsize/2);
        line(len, 0, len-arrowsize, -arrowsize/2);
        popMatrix();
      }
    }
}

//tracé de la carte d'intensité en fausses couleurs. 
void map_champE() {  
  int R, G, B;
  float z;
  color c1;
  strokeWeight(2);
  for (int i=0; i<width; i+=2) {
    for (int j=0; j<height; j+=2) {
      z=map(champE(i,j).mag(),0,0.002,0,1);
      c1=echelleCouleur(z);
      stroke(c1);
      point(i, j);
    }
  }
}


color echelleCouleur(float z) {
  float b;
  float R, G, B;
  if (z<=0.25) {
    b=0;
  } else if (z>=0.57) {
    b=1;
  } else {
    b=z/0.32-0.78125;
  }
  R=255*b;
  if (z<=0.42) {
    b=0;
  } else if (z>=0.92) {
    b=1;
  } else {
    b=2*z-0.84;
  }
  G=255*b;
  if (z<=0.42) {
    b=4*z;
  } else if (z>=0.92) {
    b=z/0.08-11.5;
  } else {
    b=-2*z+1.84;
  }
  B=255*b;
  return color(R, G, B,170);
}

color echelleCouleur2(float z) {
  float R, G, B;
  R=int(255*pow(z, 0.25));
  G=int(255*z*z*z);
  B=int(255*sin(2*PI*z));
  return color(R, G, B);
}


void affiche_control() {
   textSize(12);
   textAlign(LEFT,TOP);
   fill(255);
   textLeading(14);
   text("Clic to draw a field line \nPress space bar to display intensity map \nPress R to reset\nPress I to hide panel",14,14);
 }


void resetA(){
	for (int i = lignes.size()-1; i >= 0; i--) {lignes.remove(i)};
}

void changeA(){
	afficheMagnitude=!afficheMagnitude;
}

void keyPressed() {
  if (key == ' ') {afficheMagnitude=!afficheMagnitude;}
  if (key == 'R' || key == 'r') {
  	for (int i = lignes.size()-1; i >= 0; i--) {lignes.remove(i)};
    for (int i = equipot.size()-1; i >= 0; i--) {equipot.remove(i);}   
	} 
  if (key == 'I' || key == 'i'){afficheControl=!afficheControl;} 
}


void mousePressed() {
  if (mouseButton == LEFT) {
    // une nouvelle ligne de champ est ajoutée à la liste (à la fin par défaut)
    lignes.add(new LigneDeChamp(mouseX, mouseY, 2));
  } 
  if (mouseButton == RIGHT) {
     equipot.add(new Equipotentielle(mouseX, mouseY, 4));
  }
}


class Equipotentielle {
  float x0;
  float y0;
  float pas;

  Equipotentielle(float posX, float posY, int ppas) {
    x0=posX;
    y0=posY;
    pas= ppas;
  }
  //méthode RK2
  void display() {
    strokeWeight(1);
    stroke(255,157,0);
    float xx1, yy1;
    float angle;
    int compteur=0;
    boolean stop=false;
    xx1=x0;
    yy1=y0;
    while (!stop) {
      angle=champ(xx1, yy1).heading2D()+HALF_PI;
      angle=champ(xx1+.5*pas*cos(angle), yy1+.5*pas*sin(angle)).heading2D()+HALF_PI;
      if (compteur%2==0) {
        line(xx1, yy1, xx1+pas*cos(angle), yy1+pas*sin(angle));
      }
      compteur++;
      xx1+=pas*cos(angle);
      yy1+=pas*sin(angle);
      if (dist(OX,OY,xx1, yy1)<10 || dist(OX,OY,xx1, yy1)>width) {
        stop=true;
      }
    }
    stop=false;
    xx1=x0;
    yy1=y0;
    while (!stop) {
      angle=champ(xx1, yy1).heading2D()+HALF_PI;
      angle=champ(xx1-.5*pas*cos(angle), yy1-.5*pas*sin(angle)).heading2D()+HALF_PI;
      if (compteur%2==0) {
        line(xx1, yy1, xx1-pas*cos(angle), yy1-pas*sin(angle));
      }
      compteur++;
      xx1-=pas*cos(angle);
      yy1-=pas*sin(angle);
      if (dist(OX,OY,xx1, yy1)<10 || dist(OX,OY,xx1, yy1)>width) {
        stop=true;
      }
    }
  }
}

class LigneDeChamp {
  float x0;
  float y0;
  float pas;
  
LigneDeChamp(float posX, float posY, int ppas) {
    x0=posX;
    y0=posY;
    pas= ppas;
  }


//méthode RK2
void display() {
  strokeWeight(2);
  stroke(0,153,204,150);
  fill(0,153,204,150);
  float xx1,yy1,xx2,yy2;
  float angle;
  xx1=x0;
  yy1=y0;
  xx2=x0;
  yy2=y0;
  pushMatrix();
  translate(xx1,yy1);
  rotate(champ(xx1,yy1).heading2D());
  beginShape(TRIANGLES);
  vertex(0,3);
  vertex(8,0);
  vertex(0,-3);
  endShape();
  rotate(-champ(xx1,yy1).heading2D());
  while(!StopTrace(xx1,yy1)){
    angle=champ(xx1,yy1).heading2D();
    angle=champ(xx1+.5*pas*cos(angle),yy1+.5*pas*sin(angle)).heading2D();
    xx1+=pas*cos(angle);
    yy1+=pas*sin(angle);
    line(0,0,pas*cos(angle),pas*sin(angle));
    translate(pas*cos(angle),pas*sin(angle));
  }
  popMatrix();
  pushMatrix();
  translate(xx2,yy2);
  while(!StopTrace(xx2,yy2)){
    angle=champ(xx2,yy2).heading2D();
    angle=champ(xx2+.5*pas*cos(angle+PI),yy2+.5*pas*sin(angle+PI)).heading2D();
    xx2+=pas*cos(angle+PI);
    yy2+=pas*sin(angle+PI);
    line(0,0,pas*cos(angle+PI),pas*sin(angle+PI));
    translate(pas*cos(angle+PI),pas*sin(angle+PI));    
  }
  popMatrix();
}

Boolean StopTrace(float x,float y){
  if(abs(x-OX)<width && abs(y-OY)<height && dist(x,y,OX,OY)>10){return false;}
  else{return true;}
}
}